/****************************************************************************
 * This source file was written by Acorn Computers Limited. It is part of   *
 * the "DrawFile" library for rendering RISCOS Draw files from applications *
 * in C. It may be used freely in the creation of programs for Archimedes.  *
 * It should be used with Acorn's C Compiler Release 2 or later.            *
 *                                                                          *
 * No support can be given to programmers using this code and, while we     *
 * believe that it is correct, no correspondence can be entered into        *
 * concerning behaviour or bugs.                                            *
 *                                                                          *
 * Upgrades of this code may or may not appear, and while every effort will *
 * be made to keep such upgrades upwards compatible, no guarantees can be   *
 * given.                                                                   *
 ***************************************************************************/

/* -> h.DrawErrors
 *
 * Definition of error codes and standard messages for the DrawFile module
 * History:
 * Version 0.0 : 22 Nov 88, DAHE: created
 *         0.0a: 23 Nov 88, DAHE: first working version
 *         0.1 : 30 Nov 88, DAHE: released
 *
 * For each error, a code and the standard message are listed.
 * See h.DrawLevel0 for how to use the errors
 *
 */


#define Draw_BadObject       1 /* 'Bad object' */
#define Draw_BadObjectHandle 2 /* 'Bad object handle' */
#define Draw_TooManyFonts    3 /* 'Too many font definitions' */

#define Draw_BBoxWrong       101 /* 'Bounding box coordinates are in the wrong order' */
#define Draw_BadCharacter    102 /* 'Bad character in string' */
#define Draw_ObjectTooSmall  103 /* 'Object size is too small' */
#define Draw_ObjectTooLarge  104 /* 'Object size is too large' */
#define Draw_ObjectNotMult4  105 /* 'Object size is not a multiple of 4' */
#define Draw_ObjectOverrun   106 /* 'Object data is larger than specified size' */
#define Draw_ManyFontTables  107 /* 'There is more than one font table' */
#define Draw_LateFontTable   108 /* 'The font table appears after text object(s)' */
#define Draw_BadTextStyle    109 /* 'Bad text style word' */
#define Draw_MoveMissing     110 /* 'Path must start with a move' */
#define Draw_BadPathTag      111 /* 'Path contains an invalid tag' */
#define Draw_NoPathElements  112 /* 'Path does not contain any line or curve elements' */
#define Draw_PathExtraData   113 /* 'There is extra data present at the end of a path object' */
#define Draw_BadSpriteSize   114 /* 'The sprite definition size is inconsistent with the object size' */
#define Draw_BadTextColumnEnd 115 /* 'Missing end marker in text columns' */
#define Draw_ColumnsMismatch 116 /* 'Actual number of columns in a text area object does not match specified number of columns' */
#define Draw_NonZeroReserved 117 /* 'Non-zero reserved words in a text area object' */
#define Draw_NotDrawFile     118 /* 'This is not a Draw file' */
#define Draw_VersionTooHigh  119 /* 'Version number too high' */
#define Draw_BadObjectType   120 /* 'Unknown object type' */
#define Draw_CorruptTextArea 121 /* 'Corrupted text area (must start with '\!'') */
#define Draw_TextAreaVersion 121 /* 'Text area version number is wrong or missing' */
#define Draw_MissingNewline  122 /* 'Text area must end with a newline character' */
#define Draw_BadAlign        123 /* 'Text area: bad \A code(must be L, R, C or D)' */
#define Draw_BadTerminator   124 /* 'Text area: bad number or missing terminator' */
#define Draw_ManyDCommands   125 /* 'Text area: more than one \D command' */
#define Draw_BadFontNumber   126 /* 'Text area: bad font number' */
#define Draw_UnexpectedCharacter 127 /* 'Text area: unexpected character in \F command' */
#define Draw_BadFontWidth    128 /* 'Text area: bad or missing font width in \F command' */
#define Draw_BadFontSize     129 /* 'Text area: bad or missing font size in \F command' */
#define Draw_NonDigitV       130 /* 'Text area: non-digit in \V command' */
#define Draw_BadEscape       131 /* 'Text area: bad escape sequence' */
#define Draw_FewColumns      133 /* 'Text area must have at least one column' */
#define Draw_TextColMemory   134 /* 'Out of memory when building text area'
                                 [location field is always 0 for this error] */
#define Draw_BadJPEGSize     135 /* 'The JPEG definition size is inconsistent with the object size' */

