

#include <stdafx.h>

#include <stdio.h>
#include <math.h>


#include "os.h"
#include "flex.h"


#include "etc.h"
#include "transform.h"
#include "alloc.h"
#include "xmath.h"
#include "bezierarc.h"

#include "deb.h"

#include "drawcurves.h"










/* find the height of point (px,py) over
   straight line thru (ax,ay) and (bx,by). 
*/


#ifdef NEVER

static int height(int px,int py,int ax,int ay,int bx,int by,int tolerance)
{
	int det;
 int length;

	det=px*ay+ax*by+bx*py-ay*bx-by*px-py*ax;

	length=((bx-ax)*(bx-ax)+(by-ay)*(by-ay));

 return(tolerance*tolerance*length>det*det);

//	if(length < 0.000001)   /* height doesn't make sense, hence: */
//		return( sqrt( (px-ax)*(px-ax) + (py-ay)*(py-ay) ) );
//	else
//	return (det/length);

}


#endif




void bezdivide(bezier * p,bezier * a,bezier * b)
{
 int temp;

 a->left=p->left;
 b->right=p->right;

  /* x coords */

 temp=(p->leftc.x+p->rightc.x)/2;

 a->leftc.x =(p->left.x+p->leftc.x)/2;
 a->rightc.x=(temp+a->leftc.x)/2;
 b->rightc.x=(p->rightc.x+p->right.x)/2;
 b->leftc.x =(temp+b->rightc.x)/2;

 a->right.x=(a->rightc.x+b->leftc.x)/2;

  /* y coords */

 temp=(p->leftc.y+p->rightc.y)/2;

 a->leftc.y =(p->left.y+p->leftc.y)/2;
 a->rightc.y=(temp+a->leftc.y)/2;
 b->rightc.y=(p->rightc.y+p->right.y)/2;
 b->leftc.y =(temp+b->rightc.y)/2;

 a->right.y=(a->rightc.y+b->leftc.y)/2;

 b->left=a->right;
}



static int bdepth;
//static int bdepthmax;


void bezflatten(bezier * p,int flatness,bezflatfn fn,void * data)
{
 double   dx;
 double   dy;
 double   axby;
 double   aybx;
 double   det1;
 double   det2;
 double   length;
 double   flatness2;
 bezier   a;
 bezier   b;


 bdepth++;
// if(bdepth>bdepthmax) bdepthmax++;
 if(bdepth>20)
 {
  bdepth--;

  cprintf("flatten stack limit");
  return;
 }

 dx=p->right.x-p->left.x;
 dy=p->right.y-p->left.y;

 length=dx*dx+dy*dy;
 flatness2=(double)flatness*(double)flatness;
 length*=flatness2;

 axby=(double)p->left.x*(double)p->right.y;
 aybx=(double)p->right.x*(double)p->left.y;

	det1=(double)p->leftc.x*(double)p->left.y+axby+(double)p->right.x*(double)p->leftc.y-aybx-(double)p->right.y*(double)p->leftc.x-(double)p->leftc.y*(double)p->left.x; 
 det2=(double)p->rightc.x*(double)p->left.y+axby+(double)p->right.x*(double)p->rightc.y-aybx-(double)p->right.y*(double)p->rightc.x-(double)p->rightc.y*(double)p->left.x; 

 det1*=det1;
 det2*=det2;


// printf("length %e det1 %e det2 %e\r\n",length,det1,det2);

// cprintf("%d %d %d %d %d %d %d %d",p->left.x,p->left.y,p->leftc.x,p->leftc.y,
//                                   p->rightc.x,p->rightc.y,p->right.x,p->right.y);

 if((length>det1 && length>det2) || length<flatness)
 {
  fn(p->left.x,p->left.y,p->right.x,p->right.y,data);
 }
 else
 {
  bezdivide(p,&a,&b);
  bezflatten(&a,flatness,fn,data);
  bezflatten(&b,flatness,fn,data);
 }

 bdepth--;
}





static bezlinefn bezlf;



static os_error * lineflat(int x0,int y0,int x1,int y1,void * data)
{
 int disc;
 int dx;
 int dy;
 int x;
 int y;

 dx=(x1-x0);
 dy=(y1-y0);


 x=x0;
 y=y0;

 disc=0;

 if(dx<0)
 {
  dx=-dx;
  if(dy>0)
  {
   while(x>x1)
   {
    x--;
    disc+=dy;
    if(disc>=dx)
    {
     do
     {
      disc-=dx;
      y++;
      bezlf(x,y);
     }while(disc>=dx);
    }
    else
    {
     bezlf(x,y);
    }
   }
  }
  else
  {
   dy=-dy;
   while(x>x1)
   {
    x--;
    disc+=dy;
    if(disc>=dx)
    {
     do
     {
      disc-=dx;
      y--;
      bezlf(x,y);
     }while(disc>=dx);
    }
    else
    {
     bezlf(x,y);
    }
   }
  }
 }
 else
 {
  if(dy>0)
  {
   while(x<x1)
   {
    x++;
    disc+=dy;
    if(disc>=dx)
    {
     do
     {
      disc-=dx;
      y++;
      bezlf(x,y);
     } while(disc>=dx);
    }
    else
    {
     bezlf(x,y);
    }
   }
  }
  else
  {
   dy=-dy;
   while(x<x1)
   {
    x++;
    disc+=dy;
    if(disc>=dx)
    {
     do 
     {
      disc-=dx;
      y--;
      bezlf(x,y);
     }while(disc>=dx);
    }
    else
    {
     bezlf(x,y);
    }
   }
  }
 }

 return(NULL);

 USE(data);
}



void bezline(bezier * p,int flatness,bezlinefn fn)
{


 bezlf=fn;

 bezlf(p->left.x,p->left.y);

 bezflatten(p,flatness,lineflat,NULL);

}


static int bincx;
static int bincy;


static os_error * lineflat2(int x0p,int y0p,int x1p,int y1p,void * data)
{
 int disc;
 int dx;
 int dy;
 int x;
 int y;
 int x0;
 int x1;
 int y0;
 int y1;

/*
 dx=(x1p-x0p);
 dy=(y1p-y0p);


 if(dx!=0)
 {
  x0=((x0p+bincx-1)/bincx)*bincx;
  y0=y0p+((x0-x0p)*dy)/dx;

  x1=(x1p/bincx)*bincx;
  y1=y0p+((x1-x0p)*dy)/dx;
 }
 else
 {
  x0=x0p;
  x1=x1p;
  y0=y0p;
  y1=y1p;
 }
*/


 x0=((x0p)/bincx)*bincx;
 x1=((x1p)/bincx)*bincx;
 y0=((y0p)/bincy)*bincy;
 y1=((y1p)/bincy)*bincy;



 dx=(x1-x0)*bincy;
 dy=(y1-y0)*bincx;


 x=x0;
 y=y0;

 disc=0;

 if(x0!=x0p) bezlf(x,y);

 if(dx<0)
 {
  dx=-dx;
  if(dy>0)
  {
   while(x>x1)
   {
    x-=bincx;
    disc+=dy;
    if(disc>=dx)
    {
     do
     {
      disc-=dx;
      y+=bincy;
      bezlf(x,y);
     }while(disc>=dx);
    }
    else
    {
     bezlf(x,y);
    }
   }
  }
  else
  {
   dy=-dy;
   while(x>x1)
   {
    x-=bincx;
    disc+=dy;
    if(disc>=dx)
    {
     do
     {
      disc-=dx;
      y-=bincy;
      bezlf(x,y);
     }while(disc>=dx);
    }
    else
    {
     bezlf(x,y);
    }
   }
  }
 }
 else
 {
  if(dy>0)
  {
   while(x<x1)
   {
    x+=bincx;
    disc+=dy;
    if(disc>=dx)
    {
     do
     {
      disc-=dx;
      y+=bincy;
      bezlf(x,y);
     } while(disc>=dx);
    }
    else
    {
     bezlf(x,y);
    }
   }
  }
  else
  {
   dy=-dy;
   while(x<x1)
   {
    x+=bincx;
    disc+=dy;
    if(disc>=dx)
    {
     do 
     {
      disc-=dx;
      y-=bincy;
      bezlf(x,y);
     }while(disc>=dx);
    }
    else
    {
     bezlf(x,y);
    }
   }
  }
 }

 return(NULL);

 USE(data);
}



void bezline2(bezier * p,int flatness,bezlinefn fn,int incx,int incy)
{


 bezlf=fn;
 bincx=incx;
 bincy=incy;

 bezlf(p->left.x,p->left.y);

 bezflatten(p,flatness,lineflat2,NULL);

}


//static int bdcount;

#define DCHUNK 0x400
#define DEXTRA 8


typedef struct
{
 int ** dest;
 int    n;

} dflatstr;


static os_error * drawflat(int x0,int y0,int x1,int y1,void * data)
{
 os_error * err;
 dflatstr * df;

 df=(dflatstr*)data;

// bdcount++;

 err=schunk((flex_ptr)df->dest,(df->n+DEXTRA)*sizeof(int),DCHUNK);
 if(!err)
 {
  (*df->dest)[df->n++]=DRAWDRAW;
  (*df->dest)[df->n++]=x1;
  (*df->dest)[df->n++]=y1;
 }

 return(err);

 USE(x0);
 USE(y0);
}



os_error * draw_flatten(int * s,int ** dest,int flatness)
{
 os_error * err;
 int      * p;
 int        over;
 int        lx=0;
 int        ly=0;
 bezier     b;
 dflatstr   df;

 over=0;
 p=s;
 df.n=0;
 df.dest=dest;

 if(flatness<5) flatness=5;


// cprintf("flatness %d",flatness);
 
 // bdepthmax=0;
// bdcount=0;

 err=salloc((flex_ptr)dest,DCHUNK);
 if(!err)
 {  
  while(!over)
  {
   err=schunk((flex_ptr)df.dest,(df.n+DEXTRA)*sizeof(int),DCHUNK);
   if(err) break;
   
   switch(*p++)
   {
    case     DRAWEND:
                     (*df.dest)[df.n++]=DRAWEND;
                     over=1;
                     break;

    case    DRAWMOVE:
    case DRAWMOVESUB:
				case	DRAWMOVEGAP:
                     (*dest)[df.n++]=DRAWMOVE;
                     lx=(*df.dest)[df.n++]=*p++;
                     ly=(*df.dest)[df.n++]=*p++;
                     break;

    case    DRAWDRAW:
                     (*df.dest)[df.n++]=DRAWDRAW;
                     lx=(*df.dest)[df.n++]=*p++;
                     ly=(*df.dest)[df.n++]=*p++;
                     break;

    case   DRAWCURVE:
                     b.left.x=lx;
                     b.left.y=ly;
                     b.leftc.x=*p++;
                     b.leftc.y=*p++;
                     b.rightc.x=*p++;
                     b.rightc.y=*p++;
                     lx=b.right.x=*p++;
                     ly=b.right.y=*p++;

                     bezflatten(&b,flatness,drawflat,(void*)&df);
                     break;

    case   DRAWCLOSE:
			case	DRAWCLOSEGAP:
                     (*df.dest)[df.n++]=DRAWCLOSE;
                     break;
   }
  }
 }

// cprintf("bdmax %d bdcount %d",bdepthmax,bdcount);

 return(err);
}




os_error * draw_dash(int * s,int ** dest,int * dash)
{
 os_error * err;
 int      * p;
 int        over;
 int        lx=0;
 int        ly=0;
 dflatstr   df;
 int      * cdash;
 int      * sdash;
 int      * edash;
 int        mark;
 int        cdistance;
 int        cx;
 int        cy;
 int        dx;
 int        dy;
 int        length;
 int        size;
 int        min;
 int        lastn;
	int        state;
	int        startsubpath;
	int        ssx=0;
	int        ssy=0;

 sdash=dash+2;
 edash=sdash+dash[1];
 cdash=sdash+dash[0];
 mark=1;
 cdistance=*cdash;

 over=0;
 p=s;
 df.n=0;
 df.dest=dest;
 lastn=-1;
	startsubpath=0;

 err=salloc((flex_ptr)dest,DCHUNK);
 if(!err)
 {  
  while(!over && !err)
  {
   err=schunk((flex_ptr)df.dest,(df.n+DEXTRA)*sizeof(int),DCHUNK);
   if(err) break;
   
   switch(*p++)
   {
    case     DRAWEND:
                     lastn=df.n;
                     (*df.dest)[df.n++]=DRAWEND;
																						cprintf("drawend");
                     over=1;
                     break;

    case    DRAWMOVE:
    case DRAWMOVESUB:
																					startsubpath=1;

				case	DRAWMOVEGAP:
                     lx=*p++;
                     ly=*p++;
                     if(lastn>=0 && (*df.dest)[lastn]==DRAWMOVE)
                     {
                      (*df.dest)[lastn+1]=lx;
                      (*df.dest)[lastn+2]=ly;
																						cprintf("back gap %d %d",lx,ly);
                     }
                     else
                     {
                      lastn=df.n;
                      (*dest)[df.n++]=DRAWMOVE;
																						cprintf("drawmove %d %d",lx,ly);
                      (*df.dest)[df.n++]=lx;
                      (*df.dest)[df.n++]=ly;
                     }

                     break;

    case    DRAWDRAW:
																					if(startsubpath)
																					{
																						ssx=lx;
																						ssy=ly;
																						startsubpath=0;
																					}

																					state=0;

																					while(1)
																					{
																						if(state==1)
																						{
																							cx=ssx;
																							cy=ssy;
																						}
																						else
																						{
                       cx=*p++;
                       cy=*p++;
																						}
                      dx=(cx-lx);
                      dy=(cy-ly);
                      length=size=pythag(dx,dy);

                      while(length)
                      {
                       err=schunk((flex_ptr)df.dest,(df.n+DEXTRA)*sizeof(int),DCHUNK);
                       if(err) break;

                       min=MIN(length,cdistance);
												 
                       lx+=scale(min,dx,size);
                       ly+=scale(min,dy,size);

                       if(!mark && lastn>=0 && (*df.dest)[lastn]==DRAWMOVE)
                       {
                        (*df.dest)[lastn+1]=lx;
                        (*df.dest)[lastn+2]=ly;
																								cprintf("back %d %d",lx,ly);
                       }
                       else
                       {
                        lastn=df.n;
                        (*df.dest)[df.n++]=mark?DRAWDRAW:DRAWMOVEGAP;
																								cprintf("%s %d %d",mark?"drawdraw":"drawmovegap",lx,ly);
                        (*df.dest)[df.n++]=lx;
                        (*df.dest)[df.n++]=ly;
                       }

                       length-=min;
                       cdistance-=min;
                       if(cdistance<=0)
                       {
                        cdash++;
                        if(cdash>=edash) cdash=sdash;
                        cdistance=*cdash;
                        mark^=1;
                       }
                      }

                      lx=cx;
                      ly=cy;

																						if(state==0)
																						{
																							if(*p==DRAWCLOSE) state=1;
																							else              break;
																						}
																						else
																						{
																							break;
																						}
																					}
                     break;

    case   DRAWCURVE:
                     p+=6;
                     break;

    case   DRAWCLOSE:
   case	DRAWCLOSEGAP:
                     lastn=df.n;
                     (*df.dest)[df.n++]=DRAWCLOSE;
																					cprintf("draw close");
                     break;
   }
  }
 }


 return(err);
}




static os_error * dfw(dflatstr * df,coordstr * cx,int op)
{
 os_error * err;

 err=schunk((flex_ptr)df->dest,(df->n+DEXTRA+8)*sizeof(int),DCHUNK);
 if(!err)
 {
  (*(df->dest))[df->n++]=op;
  (*(df->dest))[df->n++]=cx->x;
  (*(df->dest))[df->n++]=cx->y;
//  cprintf("dfw %d %d",cx->x,cx->y);

 }

 return(err);
}



static os_error * dfc(dflatstr * df,coordstr * cx,int radius)
{
 os_error * err;
	coordstr   cy[13];
	int        i;

 err=schunk((flex_ptr)df->dest,(df->n+DEXTRA+31)*sizeof(int),DCHUNK);
 if(!err)
	{
  bezier_arc_circle(*(bezier_arc_coord*)cx,radius,(bezier_arc_coord*)&cy);

  (*(df->dest))[df->n++]=DRAWMOVE;
  (*(df->dest))[df->n++]=cy[0].x;
  (*(df->dest))[df->n++]=cy[0].y;

		for(i=1;i<13;i+=3)
		{
   (*(df->dest))[df->n++]=DRAWCURVE;
   (*(df->dest))[df->n++]=cy[i].x;
   (*(df->dest))[df->n++]=cy[i].y;
   (*(df->dest))[df->n++]=cy[i+1].x;
   (*(df->dest))[df->n++]=cy[i+1].y;
   (*(df->dest))[df->n++]=cy[i+2].x;
   (*(df->dest))[df->n++]=cy[i+2].y;
		}
	}

	return(err);
}








/*
		caps

		word 0 byte 0 join style 0=mitre 1=round 2=bevel
														1	leading cap 0=butt 1=round 2=projecting square 3=triangle
														2	trailing cap
														3	unused
							4						0,1 fraction of mitre limit
														2,3 integer part of mitre
							8						0,1 leading triangle cap width on each side in 1/256ths
														2,3 cap height
						12						all trailing cap triangle values

						only access 4-12 for given style

						mitre limit (length from join on one side of line to other)/thickness
						if exceeded use bevel join
*/



#define BEVEL 2
#define ROUND 1
#define MITRE 0

#define CBUTT     0
#define CROUND    1
#define CSQUARE   2
#define CTRIANGLE 3


static os_error * dfcap(dflatstr * df,int tcap,int x,int y,int * caps,int thickness,int length,int dx,int dy)
{
	os_error * err;
	coordstr   temp;
	coordstr   cx[2][2];
 int        tx;
	int        ty;
	int        wfactor;


	err=NULL;

//	cprintf("tcap %d",tcap);


	if(tcap==CROUND)
	{
		temp.x=x;
		temp.y=y;


	 dfc(df,&temp,thickness);

	}
 else
	if(tcap==CSQUARE)
	{
//		cprintf("thick %d dx %d dy %d length %d",thickness,dx,dy,length);

  tx=scale(thickness,dy,length);
  ty=scale(thickness,dx,length);

  cx[0][0].x=x-tx;
  cx[0][1].x=x+tx;
  cx[1][0].x=x-tx+ty;
  cx[1][1].x=x+tx+ty;
    
  cx[0][0].y=y+ty;
  cx[0][1].y=y-ty;
  cx[1][0].y=y+ty+tx;
  cx[1][1].y=y-ty+tx;


 	dfw(df,&cx[0][0],DRAWMOVE);
  dfw(df,&cx[0][1],DRAWDRAW);
  dfw(df,&cx[1][1],DRAWDRAW);
  dfw(df,&cx[1][0],DRAWDRAW);
  dfw(df,&cx[0][0],DRAWDRAW);
  (*df->dest)[df->n++]=DRAWCLOSE;

//		cprintf("sq cap %d %d tx %d ty %d",x,y,tx,ty);
//		cprintf("%d %d %d %d",x-tx,x+tx,x-tx+ty,x+tx+ty);
//		cprintf("%d %d %d %d",y+ty,y-ty,y+ty+tx,y-ty+tx);
	}
	else
	if(tcap==CTRIANGLE)
	{
		wfactor=*caps;
		wfactor&=0xFFFF;

  tx=scale((thickness*wfactor)/128,dy,length);
  ty=scale((thickness*wfactor)/128,dx,length);

		cprintf("tx %d thickness %d wfac %d dx %d length %d",tx,thickness,wfactor,dx,length);
		cprintf("ty %d thickness %d wfac %d dy %d length %d",ty,thickness,wfactor,dy,length);


  cx[0][0].x=x-tx;
  cx[0][1].x=x+tx;
  cx[0][0].y=y+ty;
  cx[0][1].y=y-ty;

		wfactor=(*caps)>>16;
		wfactor&=0xFFFF;

  tx=scale((thickness*wfactor)/128,dy,length);
  ty=scale((thickness*wfactor)/128,dx,length);

  cx[1][0].x=x+ty;
  cx[1][0].y=y+tx;


 	dfw(df,&cx[0][0],DRAWMOVE);
  dfw(df,&cx[0][1],DRAWDRAW);
  dfw(df,&cx[1][0],DRAWDRAW);
  dfw(df,&cx[0][0],DRAWDRAW);
  (*df->dest)[df->n++]=DRAWCLOSE;
	}


	return(err);
}



os_error * draw_thicken(int * s,int ** dest,int thickness,int * caps)
{
 os_error * err;
 int      * p;
 int        over;
 int        lx=0;
 int        ly=0;
 dflatstr   df;

 int        crx=0;
 int        cry=0;
 int        dx=0;
 int        dy=0;
 int        length=0;
 int        tx;
 int        ty;
 coordstr   cx[4][2];
 int        first;
	int        join;
 int        lcap;
	int        tcap;
	double     lt=NULL;
	int        xjoin;
	double     turnangle;
	double     mitreangle;
	int        startsubpath;
	int								ssx=0;
	int								ssy=0;
	int								sscx=0;
	int								sscy=0;
	int        state;
	int								crx2=0;
	int	 						cry2=0;
	int								ssx2=0;
	int								ssy2=0;
	int								sscx2=0;
	int								sscy2=0;
	int        length2=0;
	int        dx2=0;
	int        dy2=0;

// int        element=0;


	cprintf("thicken");

 over=0;
 p=s;

 df.n=0;
 df.dest=dest;

	join=(caps[0] & 0xFF);
	lcap=((caps[0]>>8) & 0xFF);
	tcap=((caps[0]>>16) & 0xFF);
	startsubpath=1;

	if(join==MITRE)
 {
 	lt=((double)0x10000)/((double)caps[1]);
		lt=2*asin(lt);

//		cprintf("mitre lt %f",lt);
	}


 first=1;
 state=0;

 thickness/=2;

 err=salloc((flex_ptr)dest,DCHUNK);
 if(!err)
 {  
  if(!err)
  {
   while(!over && !err)
   {
    err=schunk((flex_ptr)df.dest,(df.n+DEXTRA+8)*sizeof(int),DCHUNK);
    if(err) break;
   
//    element&=~0xFFFF;
//    element+=0x10000;

    switch(*p++)
    {
     case     DRAWEND:
                      (*df.dest)[df.n++]=DRAWEND;
                      over=1;
                      break;


     case    DRAWMOVE:		// start sub path
     case DRAWMOVESUB:
																						startsubpath=1;
 				case	DRAWMOVEGAP:
                      lx=*p++;
                      ly=*p++;

//                      cprintf("mv x %x y %x",lx,ly);

                      (*dest)[df.n++]=DRAWMOVE;
                      (*df.dest)[df.n++]=lx;
                      (*df.dest)[df.n++]=ly;

                      break;

     case    DRAWDRAW:
                      first=1;
																						state=0;

                      while(1)
                      {
//                       err=schunk((flex_ptr)df.dest,(df.n+DEXTRA+8)*sizeof(int),DCHUNK);
//                       if(err) break;
//                       not needed because in dfw()

//                       element++;

//                       if((element & 0xFFFF)<3)
//                       {
//                        cprintf("state %d x %d y %d",state,*p,*(p+1));
//
//                       }

																							if(state==0)
																							{
//                        cprintf("dr crx %x cry %x",crx,cry);

                        crx=*p++;
                        cry=*p++;
																							}
																							else
																							if(state==1)
																							{
																								crx=ssx;
																								cry=ssy;
																							}
																							else
																							if(state==2)
																							{
																								crx=sscx;
																								cry=sscy;
																							}

                       dx=(crx-lx);
                       dy=(cry-ly);

																							if(startsubpath)
																							{
																								ssx=lx;
																								ssy=ly;
																								sscx=crx;
																								sscy=cry;
																								startsubpath=0;
																							}

                       length=pythag(dx,dy);
                       if(length==0)
                       {

                       
                       }
																							else
																							{
																								crx2=crx;
																								cry2=cry;
																								ssx2=ssx;
																								ssy2=ssy;
																								sscx2=sscx;
																								sscy2=sscy;
																								length2=length;
																								dx2=dx;
																								dy2=dy;

                        tx=scale(thickness,dy,length);
                        cx[2][0].x=lx-tx;
                        cx[2][1].x=lx+tx;
                        cx[3][0].x=crx-tx;
                        cx[3][1].x=crx+tx;
    
                        ty=scale(thickness,dx,length);
                        cx[2][0].y=ly+ty;
                        cx[2][1].y=ly-ty;
                        cx[3][0].y=cry+ty;
                        cx[3][1].y=cry-ty;


			 																				if(state!=2)
				 																			{
                         dfw(&df,&cx[2][0],DRAWMOVE);
                         dfw(&df,&cx[2][1],DRAWDRAW);
                         dfw(&df,&cx[3][1],DRAWDRAW);
                         dfw(&df,&cx[3][0],DRAWDRAW);
                         dfw(&df,&cx[2][0],DRAWDRAW);
                         (*df.dest)[df.n++]=DRAWCLOSE;
					 																		}

//  if this is not first time, sort out join                    


                        if(!first)
                        {
						 																		int dxf;
						 																		int dyf;
						 																		int dxs;
						 																		int dys;
						 																		double tf;
						 																		double ts;
							 																	coordstr temp;
						 																		int      mitre;

						 																		dxf=cx[1][0].x-cx[0][0].x;
						 																		dyf=cx[1][0].y-cx[0][0].y;
						 																		dxs=cx[3][0].x-cx[2][0].x;
						 																		dys=cx[3][0].y-cx[2][0].y;

								 																tf=atan2((double)dyf,(double)dxf);
								 																ts=atan2((double)dys,(double)dxs);
								 																turnangle=(ts-tf);
								 																if(turnangle>PI) turnangle-=2*PI;
								 																if(turnangle<-PI) turnangle+=2*PI;

									 															xjoin=join;

										 														if(turnangle!=0)
											 													{
											 														if(xjoin==MITRE)
											 														{
											 															if(turnangle<0) mitreangle=PI+turnangle;
											 															else												mitreangle=PI-turnangle;

//																										cprintf("angle %f",mitreangle);

											 															if(mitreangle<lt)	{xjoin=BEVEL;/*cprintf("mitre limited lt %f m %f turn %f",lt,mitreangle,turnangle);*/}
											 															else
											 															{
												 														 mitre=(int)(((double)thickness)/tan(mitreangle/2));
																											
                            if(turnangle<0)
													 														{
															 											 	temp.x=cx[2][0].x-scale(mitre,dx,length);
														 												 	temp.y=cx[2][0].y-scale(mitre,dy,length);

//																											cprintf("do mitre1 20 %d %d m %d %d 10 %d %d",cx[2][0].x,cx[2][0].y,temp.x,temp.y,cx[1][0].x,cx[1][0].y);

 															 									  	dfw(&df,&cx[1][0],DRAWMOVE);
 															 									  	dfw(&df,&temp,DRAWDRAW);
 															 									  	dfw(&df,&cx[2][0],DRAWDRAW);

                             coordstr cc;
                             cc.x=lx;
                             cc.y=ly;
                             dfw(&df,&cc,DRAWDRAW);
// 															 									  	dfw(&df,&cx[2][1],DRAWDRAW);
// 															 									  	dfw(&df,&cx[1][1],DRAWDRAW);
                             (*df.dest)[df.n++]=DRAWCLOSE;
																	 										}
																		 									else
																		 									{
																		 								 	temp.x=cx[2][1].x-scale(mitre,dx,length);
																		 								 	temp.y=cx[2][1].y-scale(mitre,dy,length);

//																											cprintf("do mitre2 20 %d %d m %d %d 10 %d %d",cx[2][0].x,cx[2][0].y,temp.x,temp.y,cx[1][0].x,cx[1][0].y);

 																		 						  	dfw(&df,&cx[1][1],DRAWMOVE);
 																		 						  	dfw(&df,&temp,DRAWDRAW);
 																		 						  	dfw(&df,&cx[2][1],DRAWDRAW);

                             coordstr cc;
                             cc.x=lx;
                             cc.y=ly;
                             dfw(&df,&cc,DRAWDRAW);

// 																		 						  	dfw(&df,&cx[2][0],DRAWDRAW);
// 																		 						  	dfw(&df,&cx[1][0],DRAWDRAW);
                             (*df.dest)[df.n++]=DRAWCLOSE;
																			 								}
																			 							}
																			 						}

 																		 						if(xjoin==BEVEL)  // can fall through here from mitre
 																		 						{
//                           cprintf("bevel angle %f",turnangle);

 																		 							if(turnangle<0)
																			 							{
 																		 						 	dfw(&df,&cx[1][0],DRAWMOVE);
 																		 						 	dfw(&df,&cx[2][0],DRAWDRAW);
                            coordstr cc;
                            cc.x=lx;
                            cc.y=ly;
                            dfw(&df,&cc,DRAWDRAW);
// 																		 						 	dfw(&df,&cx[2][1],DRAWDRAW);
// 																		 						 	dfw(&df,&cx[1][1],DRAWDRAW);
                            (*df.dest)[df.n++]=DRAWCLOSE;

//                          if((element & 0xFFFF)<3)  cprintf("bevel1 %x %d %d %d %d %d %d",element,cx[1][0].x,cx[1][0].y,cx[2][0].x,cx[2][0].y,lx,ly);
																			 							}
																				 						else
																				 						{
 																			 					 	dfw(&df,&cx[1][1],DRAWMOVE);
 																			 					 	dfw(&df,&cx[2][1],DRAWDRAW);
                            coordstr cc;
                            cc.x=lx;
                            cc.y=ly;
                            dfw(&df,&cc,DRAWDRAW);
// 																			 					 	dfw(&df,&cx[2][0],DRAWDRAW);
// 																			 					 	dfw(&df,&cx[1][0],DRAWDRAW);
                            (*df.dest)[df.n++]=DRAWCLOSE;
//                             cprintf("bevel2 %x %d %d %d %d %d %d",element,cx[1][1].x,cx[1][1].y,cx[2][1].x,cx[2][1].y,lx,ly);
																				 						}
 																			 					}
																					 				else
																						 			if(xjoin==ROUND)
																						 			{
//                           cprintf("round end");
																						 				temp.x=lx;
																						 				temp.y=ly;
																						 				dfc(&df,&temp,thickness);
																						 			}
																						 		}
                        }

                        cx[0][0]=cx[2][0];
                        cx[0][1]=cx[2][1];
                        cx[1][0]=cx[3][0];
                        cx[1][1]=cx[3][1];
																							
//  if this is the first time output leading cap

       

                        lx=crx;
                        ly=cry;
                        first=0;
																							}
// if a MOVE or a CLOSE set up

																							if(state==1)
																							{

																								state=2;
																							}
																							else
																							if(state==2)
																							{

																								break;
																							}
																							else
																							{
 																							if(*p==DRAWCLOSE)
	 																						{
		 																						state=1;
																								}
																								else
																								{
                         if(*p!=DRAWDRAW) break;
                         p++;
																								}
																							}
                      }

// output caps
																						if(state==0)		// i.e. the curve is not closed
																						{


 																					 if(length2!=0) dfcap(&df,lcap,crx2,cry2,caps+3,thickness,length2,dx2,dy2);

                       dx=(sscx2-ssx2);
                       dy=(sscy2-ssy2);
                       length=pythag(dx,dy);
																							if(length!=0) dfcap(&df,tcap,ssx2,ssy2,caps+2,thickness,length,-dx,-dy);
																						}
                      break;

     case   DRAWCURVE:
                      p+=6;
                      break;

     case   DRAWCLOSE:
	   case	DRAWCLOSEGAP:
//                      (*df.dest)[df.n++]=DRAWCLOSE;
                      break;
    }
   }
  }
 }


 return(err);
}




os_error *	draw_bounds(int * path,coordstr * cx)
{
	int   over;
 int   xmin=0;
 int   xmax=0;
 int   ymin=0;
 int   ymax=0;
	int   first;
	int * p;
	int   x;
	int   y;
	int   i;


	p=path;
	over=0;
	first=1;

 while(!over)
 {
   
  switch(*p++)
  {
     case     DRAWEND:
                      over=1;
                      break;

     case    DRAWMOVE:
     case DRAWMOVESUB:
 				case	DRAWMOVEGAP:
     case    DRAWDRAW:
                      x=*p++;
                      y=*p++;
																						if(first)
																						{
                       xmin=xmax=x;
                       ymin=ymax=y;
																							first=0;
																						}
																						else
																						{
                       if(x<xmin) xmin=x;
                       if(x>xmax) xmax=x;
                       if(y<ymin) ymin=y;
                       if(y>ymax) ymax=y;
																						}
                      break;


     case   DRAWCURVE:
 																					for(i=0;i<3;i++)
																						{
                       x=*p++;
                       y=*p++;
                       if(x<xmin) xmin=x;
                       if(x>xmax) xmax=x;
                       if(y<ymin) ymin=y;
                       if(y>ymax) ymax=y;
																						}
                      break;

     case   DRAWCLOSE:
	   case	DRAWCLOSEGAP:
                      break;
  }
 }


 cx->x=xmin;
 cx->y=ymin;
 cx++;
 cx->x=xmax;
 cx->y=ymax;


	return(NULL);
}




int draw_copy(int * src,int * dest,int destsize)
{
	int * p;
	int * q;
 int * lt;
	int   i;
	int   over;

	p=src;
	q=dest;
	lt=dest+destsize/sizeof(int);
	lt-=7;			// max size
	over=0;

	while(!over)
	{
		if(q>=lt)
		{
			*q++=DRAWEND;
			break;
		}

  switch(*q++=*p++)
  {
     case     DRAWEND:
                      over=1;
                      break;

     case    DRAWMOVE:
     case DRAWMOVESUB:
 				case	DRAWMOVEGAP:
     case    DRAWDRAW:
                      *q++=*p++;
                      *q++=*p++;
                      break;


     case   DRAWCURVE:
 																					for(i=0;i<3;i++)
																						{
                       *q++=*p++;
                       *q++=*p++;
																						}
                      break;

     case   DRAWCLOSE:
	   case	DRAWCLOSEGAP:
                      break;
  }
	}

	return(p-src);
}








#ifdef NEVER


static os_error * dfwr(dflatstr * df,coordstr * cx,int op)
{
 os_error * err;

 err=schunk((flex_ptr)df->dest,(df->n+DEXTRA+8)*sizeof(int),DCHUNK);
 if(!err)
 {

  (*(df->dest))[df->n++]=cx->y;
  (*(df->dest))[df->n++]=cx->x;
  (*(df->dest))[df->n++]=op;

  cprintf("dfwr %d %d",cx->x,cx->y);
 }

 return(err);
}




os_error * draw_thicken(int * s,int ** dest,int thickness,int * caps)
{
 os_error * err;
 int      * p;
 int        over;
 int        lx=0;
 int        ly=0;
 dflatstr   df;
 dflatstr   dr;
 int        crx;
 int        cry;
 int        dx;
 int        dy;
 int        length;
 int        tx;
 int        ty;
 coordstr   cx[4][2];
 int        first;
 int      * reverse;
 int        i;


 over=0;
 p=s;

 df.n=0;
 df.dest=dest;

 dr.n=0;
 dr.dest=&reverse;

 thickness/=2;

 err=salloc((flex_ptr)dest,DCHUNK);
 if(!err)
 {  
  err=salloc((flex_ptr)&reverse,DCHUNK);
  if(!err)
  {
   while(!over && !err)
   {
    err=schunk((flex_ptr)df.dest,(df.n+DEXTRA+8)*sizeof(int),DCHUNK);
    if(err) break;
   
    switch(*p++)
    {
     case     DRAWEND:
                      (*df.dest)[df.n++]=DRAWEND;
                      over=1;
                      break;

     case    DRAWMOVE:
     case DRAWMOVESUB:
 				case	DRAWMOVEGAP:
                      lx=*p++;
                      ly=*p++;

                      (*dest)[df.n++]=DRAWMOVE;
                      (*df.dest)[df.n++]=lx;
                      (*df.dest)[df.n++]=ly;

                      break;

     case    DRAWDRAW:
                      first=1;
                      dr.n=0;
                      while(1)
                      {
                       crx=*p++;
                       cry=*p++;
                       dx=(crx-lx);
                       dy=(cry-ly);
                       length=pythag(dx,dy);

                       tx=scale(thickness,dy,length);
                       cx[2][0].x=lx-tx;
                       cx[2][1].x=lx+tx;
                       cx[3][0].x=crx-tx;
                       cx[3][1].x=crx+tx;
    
                       ty=scale(thickness,dx,length);
                       cx[2][0].y=ly+ty;
                       cx[2][1].y=ly-ty;
                       cx[3][0].y=cry+ty;
                       cx[3][1].y=cry-ty;


//  if this is not first time, sort out join                    
//  output first section, adjust second

                       if(!first)
                       {
                        dfw(&df,&cx[1][0],DRAWDRAW);
                        dfwr(&dr,&cx[0][1],DRAWDRAW);                      
                       }

                       cx[0][0]=cx[2][0];
                       cx[0][1]=cx[2][1];
                       cx[1][0]=cx[3][0];
                       cx[1][1]=cx[3][1];

//  if this is the first time output leading cap

                       if(first)
                       {
                        dfw(&df,&cx[0][1],DRAWMOVE);


                        dfw(&df,&cx[0][0],DRAWDRAW);
                       }
                      

                       lx=crx;
                       ly=cry;

                       if(*p!=DRAWDRAW) break;
                       p++;
                       first=0;
                      }

// output trailing cap


                      dfw(&df,&cx[1][0],DRAWDRAW);
                      dfwr(&dr,&cx[0][1],DRAWDRAW);
                     // from 1 0 to 1 1
                      dfw(&df,&cx[1][1],DRAWDRAW);


                      err=schunk((flex_ptr)df.dest,(df.n+dr.n+DEXTRA+8)*sizeof(int),DCHUNK);
                      if(!err)
                      {
                       for(i=dr.n-1;i>=0;i--)
                       {
                        (*df.dest)[df.n++]=(*dr.dest)[i];
                       }

                       (*df.dest)[df.n++]=DRAWCLOSE;
                      }
                      break;

     case   DRAWCURVE:
                      p+=6;
                      break;

     case   DRAWCLOSE:
	   case	DRAWCLOSEGAP:
                      (*df.dest)[df.n++]=DRAWCLOSE;
                      break;
    }
   }
  }
 }


 return(err);
}




                      x00=lx-tx;
                      x01=lx+tx;
                      x10=cx+tx;
                      x11=cx-tx;

                      ty=scale(thickness,dx,length);
                      y00=ly+ty;
                      y01=ly-ty;
                      y10=cy-ty;
                      y11=cy+ty;

//                     cprintf("tx %d ty %d thick %d",tx,ty,thickness);

                      (*df.dest)[df.n++]=DRAWMOVE;
                      (*df.dest)[df.n++]=x00;
                      (*df.dest)[df.n++]=y00;
                   
                      (*df.dest)[df.n++]=DRAWDRAW;
                      (*df.dest)[df.n++]=x01;
                      (*df.dest)[df.n++]=y01;

                      (*df.dest)[df.n++]=DRAWDRAW;
                      (*df.dest)[df.n++]=x10;
                      (*df.dest)[df.n++]=y10;

                      (*df.dest)[df.n++]=DRAWDRAW;
                      (*df.dest)[df.n++]=x11;
                      (*df.dest)[df.n++]=y11;

                      (*df.dest)[df.n++]=DRAWCLOSE;




#endif