#include <stdafx.h>

#include <stdlib.h>
#include <stdio.h>



#include "os.h"
#include "wimp.h"

#include "osunits.h"
#include "flex.h"
#include "alloc.h"


#include "colour.h"
#include "xmath.h"

#include "etc.h"




#include "getscreen.h"





/*-------------------------------------------------------------------------- 
   CopyBitmapFromScreen: Creates GDI bitmap object and copies a rectangular 
                         are of the screen into it. Returns bitmap handle.
  --------------------------------------------------------------------------*/

static HBITMAP getscreenbitmap(RECT * prect, BOOL bIncludeCursor)
{
     HBITMAP hBitmap ;
     HDC     hdcScr, hdcMem ;

          // Check if the bitmap has a zero dimension

     if (prect->top == prect->bottom || prect->left == prect->right)
          return NULL ;

          // Don't look at me like that! This is a documented use of the
          //   GetDC call to return a device context for the display!

     hdcScr = GetDC (NULL) ;

          // Create a memory DC and a compatible bitmap

     hdcMem = CreateCompatibleDC (hdcScr) ;

     hBitmap = CreateCompatibleBitmap (hdcScr, abs (prect->right - prect->left), 
                                               abs (prect->bottom - prect->top)) ;

          // Return NULL if there's not enough memory to create the bitmap

     if (hBitmap == NULL)
     {
          ReleaseDC (NULL, hdcScr) ;
          DeleteDC (hdcMem) ;
          return NULL ;
     }

          // Select the bitmap into the memory DC and copy the screen to it
          // Note: Bitmap will be flipped if prect->left > prect->right or
          //   prect->top > prect->bottom. Use abs() or BitBlt() to avoid
          //   flipping.

     SelectObject (hdcMem, hBitmap) ;

     StretchBlt (hdcMem, 0, 0, abs (prect->right - prect->left),
                               abs (prect->bottom - prect->top), 
                 hdcScr, prect->left, prect->top, 
                               prect->right - prect->left,
                               prect->bottom - prect->top, SRCCOPY) ;

          // If the user wants to include the cursor in the image, 
          //   we must now draw the current cursor on the memory device 
          //   context. This isn't as easy as it sounds.

     if (bIncludeCursor)
     {
          DWORD   tidCursor, tidCurrent ;
          HCURSOR hCursor ;
          HWND    hwndCursor ;
          POINT   ptCursor ;

               // The following two calls for getting the cursor handle and 
               //   the cursor position would have worked in 16-bit Windows

          hCursor = GetCursor () ;      // doesn't work!
          GetCursorPos (&ptCursor) ;

               // However, in 32-bit Windows the GetCursor call returns the
               //   handle of the last cursor to appear above the thread 
               //   that calls the GetCursor function. 
               // Usually, that cursor is the window sizing cursor,

               // One possible solution is the GetCursorInfo function. 
               //   However, that function is not documented as supported
               //   under Windows 95 or versions of Windows NT 4.0 prior
               //   to SP6.
               // In reality, GetCursorInfo does not seem to work under
               //   Windows 4.0 SP6 either!

               // A better solution is to determine the thread ID under the 
               //   cursor and the current thread ID (the thread ID of this
               //   process). See Knowledge Base article Q230495.

          hwndCursor = WindowFromPoint (ptCursor) ;
          tidCursor  = GetWindowThreadProcessId (hwndCursor, NULL) ;
          tidCurrent = GetCurrentThreadId () ;

               // Then, if these two thread IDs are not equal, attach
               //   the current thread to the thread of the window
               //   underneath the cursor. This allows the GetCursor
               //   function to return the current cursor handle.

          if (tidCursor != tidCurrent)
          {
               if (AttachThreadInput (tidCurrent, tidCursor, TRUE))
               {
                    hCursor = GetCursor () ;
                    AttachThreadInput (tidCurrent, tidCursor, FALSE) ;
               }
          }

               // Now draw the cursor on the memory device context

          DrawIconEx (hdcMem, ptCursor.x - prect->left, 
                              ptCursor.y - prect->top, 
                      hCursor, 0, 0, 0, NULL, DI_DEFAULTSIZE | DI_NORMAL) ;
     }

          // Clean up and return the handle to the bitmap

     DeleteDC (hdcMem) ;
     ReleaseDC (NULL, hdcScr) ;

     return hBitmap ;
}

/*-------------------------------------------------------
   GetCurrentPalette: Obtains the current system palette.
  -------------------------------------------------------*/

static HPALETTE getcurpalette(HWND hwnd)
{
     HDC          hdc ;
     HPALETTE     hPalette ;
     int          iSize ;
     LOGPALETTE * plogpal ;

          // Get a video device context

     hdc = GetDC (hwnd) ;

          // If the video device supports palettes, get palette size
          //   (Betcha a hundred to one it'll be 256!)

     if (RC_PALETTE & GetDeviceCaps (hdc, RASTERCAPS))
     {
          iSize = GetDeviceCaps (hdc, SIZEPALETTE) ;

               // Allocate memory for the palette and get it.

          plogpal =(LOGPALETTE *) malloc (sizeof (LOGPALETTE) + 
                            sizeof (PALETTEENTRY) * (iSize - 1)) ;

          plogpal->palVersion    = 0x0300 ;
          plogpal->palNumEntries = (unsigned short)iSize ;

          GetSystemPaletteEntries (hdc, 0, iSize, plogpal->palPalEntry) ;

               // Now create a palette based on the system palette entries

          hPalette = CreatePalette (plogpal) ;

               // free the memory
     
          free (plogpal) ;
     }
          // If the device doesn't support palettes, hPalette is NULL.

     else
          hPalette = NULL ;

          // Release the DC

     ReleaseDC (hwnd, hdc) ;

     return hPalette ;
}

/*------------------------------------------------------------------------- 
   ConvertDDBtoDIB: Converts a GDI bitmap object to a DIB using a palette.
                    Function based on Knowledge Base article Q80080.
  -------------------------------------------------------------------------*/

static BYTE * DDB2DIB (HWND hwnd, HBITMAP hBitmap, HPALETTE hPalette,int & size)
{
     BITMAP       bm ;
     BITMAPINFO * pbmi ;
     BYTE       * pBits ;
     HDC          hdc ;
     WORD         wBitCount, wNumColor=0, wColorEntries, wRowLength ;

          // Get information about the DDB & calculate bits per pixel

     GetObject (hBitmap, sizeof (BITMAP), (PSTR) &bm) ;

     wBitCount = (WORD)(bm.bmPlanes * bm.bmBitsPixel) ;

          // Figure out the DIB bit count and number of colors.
          //   (Probably more generalized than it needs to be)

     if (wBitCount > 16)
     {
          wNumColor = 0 ;
          wBitCount = 24 ;
     }
     else if (wBitCount > 8)
     {
          wNumColor = 0 ;
          wBitCount = 16 ;
     }
     else if (wBitCount > 4)
     {
          wNumColor = (WORD)(1 << wBitCount) ;
          wBitCount = 8 ;
     }
     else if (wBitCount > 2)
     {
          wNumColor = (WORD)(1 << wBitCount) ;
          wBitCount = 4 ;
     }
     else if (wBitCount == 1)
     {
          wNumColor = 2 ;
     }

          // Determine the number of entries in the DIB color table

     wColorEntries = wNumColor ;

     if (wColorEntries == 0 && wBitCount <= 8)
          wColorEntries = (WORD)(1 << wBitCount) ;

          // DIB rows are always multiple of 4 bytes long.

     wRowLength = (WORD)(((bm.bmWidth * wBitCount + 31) & ~31) >> 3 );

          // Allocate memory for the entire DIB.

     size=sizeof(BITMAPINFOHEADER)+sizeof(RGBQUAD)*wColorEntries+wRowLength*bm.bmHeight;

//     pbmi =(BITMAPINFO *)malloc(size);

     salloc((flex_ptr)&pbmi,size);

     if (pbmi == NULL)
          return NULL ;

          // Initialize the BITMAPINFOHEADER

     pbmi->bmiHeader.biSize          = sizeof (BITMAPINFOHEADER) ;
     pbmi->bmiHeader.biWidth         = bm.bmWidth ;
     pbmi->bmiHeader.biHeight        = bm.bmHeight ;
     pbmi->bmiHeader.biPlanes        = 1 ;
     pbmi->bmiHeader.biBitCount      = wBitCount ;
     pbmi->bmiHeader.biCompression   = BI_RGB ;
     pbmi->bmiHeader.biSizeImage     = 0 ;
     pbmi->bmiHeader.biXPelsPerMeter = 0 ;
     pbmi->bmiHeader.biYPelsPerMeter = 0 ;
     pbmi->bmiHeader.biClrUsed       = wNumColor ;
     pbmi->bmiHeader.biClrImportant  = 0 ;

          // Determine the address of the pixel bits in the DIB

     pBits = (BYTE *) pbmi + sizeof (BITMAPINFOHEADER) + 
                             sizeof (RGBQUAD) * wColorEntries ;

          // Get a video device context and select the palette into it

     hdc = GetDC (hwnd) ;
     SelectPalette (hdc, hPalette, FALSE) ;
     RealizePalette (hdc) ;

          // Here's the function we've all been waiting for.

     GetDIBits (hdc, hBitmap, 0, bm.bmHeight, pBits, pbmi, DIB_RGB_COLORS) ;
          
          // Clean up and get out.

     ReleaseDC (hwnd, hdc) ;

     return (BYTE *) pbmi ;
}



os_error * grabscreen(int flags,int x0,int y0,int x1,int y1,char ** pdib,int * length)
{
 RECT      rect;
 HBITMAP   hBitmap;
 HPALETTE  hPalette;
 int       size;

 rect.bottom=ycoordos2pix(y0);
 rect.left=xcoordos2pix(x0);
 rect.right=xcoordos2pix(x1);
 rect.top=ycoordos2pix(y1);

 rect.left=MAX(rect.left,0);
 rect.top=MAX(rect.top,0);
 rect.right=MIN(rect.right,cvdu.xpix);
 rect.bottom=MIN(rect.bottom,cvdu.ypix);


 hBitmap=getscreenbitmap(&rect,flags & 0x1);

 if(hBitmap==NULL)
 {

 }
 else
 {
  hPalette=getcurpalette(NULL);

 // Create DIB from DDB and palette
  *pdib=(char*)DDB2DIB(NULL,hBitmap,hPalette,size);
  *length=size;

  ((BITMAPINFO*)(*pdib))->bmiHeader.biXPelsPerMeter=scale(cvdu.xdpi,10000,254);
  ((BITMAPINFO*)(*pdib))->bmiHeader.biYPelsPerMeter=scale(cvdu.ydpi,10000,254);
 }

 return(NULL);
}


